/* If you use or change this program, you must let me know.  email me at              */
/* scorey@xmission.com                                                               */
/*  REXX program to convert a cd to mp3s using cddb and tonigy.                                                               */
/*  It will also write cddb info to the EA's of the tonigy wav files and the mp3 files                                        */
/*  Requires the programs querycddb.exe, and lame.exe.                                                                        */
/*  querycddb.exe is part of audiocdcreator from http://www.geocities.com/SiliconValley/Sector/5785/creator/creatormain.htm   */
/*  lame.exe is available from http://users.ox.ac.uk/~magd1113/mpeg                                                           */
/*  querycddb.exe and lame.exe must be in the same directory as tutil.cmd (this file)                                         */
/*  I find it easiest to install audiocdcreator, and put this file and lame.exe in audiocdcreator\bin with querycddb.exe      */

CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

/* set the default options */
domp3=0
doEAs=1
copy=0
keepTempFile=0
ask=0
nogenre=0
nocddb=0
curdir=DIRECTORY()

/* Translation tables for filenames.  See REXX function Translate() */
intranslate = ':*|\/?"'
outtranslate ='-------'

/* parse the command line */

PARSE ARG commandLine
switches.0=0
arguments.0=0
arguments.1='tutil.opt'

DO WHILE commandLine >< ''
        PARSE VAR commandLine thisArg commandLine
        IF Left(thisArg, 1) = '/' | Left(thisArg, 1) = '-' THEN
                DO
                j = switches.0 + 1
                switches.j = SubStr(thisArg, 2)
                SELECT
                        WHEN switches.j ='?' | switches.j = 'h' THEN
                                DO
                                say
                                say '  If no command line parameters, the program will read'
                                say '  the configuration file tutil.opt, connect to the specified cddb'
                                say "  server, and write the CD info to the EA's of the"
                                say '  WAV files on the Tonigy drive'
                                say
                                say '  command line parameters are:'
                                say
                                say '     /mp3       convert to mp3s in dir specified in config file'
                                say '     /c         copy files to dir specified in config file'
                                say "     /noeas     don't write extended attributes to tonigy drive"
                                say "     /keep      keep the temp file cddbxxxx.tmp which contains the CD info"
                                say '     /ask       ask which track to encode (only use with /mp3 or /c)'
                                say "     /nogenre   don't include genre in the ID3 tag"
                                say "     /nocddb    don't lookup info from the cddb server.  Interactive mode"
                                say '     /? or /h   this help message'
                                say
                                say '  you can also specify a different config file on the command line:'
                                say
                                say '     tutil c:\differentconfig.file'
                                EXIT
                                END
                        WHEN switches.j = 'mp3' THEN
                                domp3 = 1
                        WHEN switches.j = 'noeas' THEN
                                doEAs = 0
                        WHEN switches.j = 'c' THEN
                                copy = 1
                        WHEN switches.j = 'keep' THEN
                                keepTempFile = 1
                        WHEN switches.j = 'nogenre' THEN
                                nogenre = 1
                        WHEN switches.j = 'nocddb' THEN
                                nocddb = 1
                        WHEN switches.j = 'ask' THEN
                                 ask = 1
                        OTHERWISE
                                DO
                                say
                                say '   /'||switches.j 'is an invalid parameter.  use /? for help'
                                EXIT
                                END
                        END
                switches.0=j
                END
        ELSE
                DO
                j = arguments.0 + 1
                IF Left(thisArg, 1) = '"' THEN
                        DO
                        commandLine = thisArg commandLine
                        PARSE VAR commandLine '"' thisArg '"' commandLine
                        END
                arguments.j = thisArg
                arguments.0 = j
                END
        END

/* check if /ask conditions are met */
IF domp3=0 & copy=0 THEN
   noask=1

IF ask=1 & noask=1 THEN
   DO
   say 'only use /ask with /mp3 or /c'
   EXIT
   END

/* try to find options file */
rc=SysFileTree(arguments.1,stem,F)
IF stem.0=0 THEN
        DO
        SAY 'options file 'arguments.1' was not found'
        EXIT
        END

/* Get the info from the options file */
I=1
DO WHILE Lines(arguments.1)
        opt.I = SubWord(LineIn(arguments.1),2)
        /* say opt.I  */
        I=I+1
END

IF opt.1\='tutil configuration file' THEN
        DO
        say
        say 'the file 'arguments.1' is not a valid configuration file'
        EXIT
        END
CDdrive=opt.2
TonigyDrive=opt.3
User=opt.4
UserHost=opt.5
cddbHost=opt.6
cddbPort=opt.7
mp3Dir=opt.8
LameOpts=opt.9


info = SysDriveInfo(TonigyDrive)
IF info >< ''THEN
        DO
        PARSE VAR info driveNm freeBytes usedBytes volLabel
        volLabel = Strip(volLabel, 'B') /* needed to properly get volume label */
        IF SubWord(volLabel, 1, 1) >< 'Tonigy' THEN
                DO
                say 'error: the drive 'TonigyDrive' specified as the'
                SAY "TonigyDrive in "arguments.1" isn't a Tonigy Drive"
                SAY "Use /? for other help"
                EXIT
                END
        END
ELSE
        DO
        SAY "error: the drive "TonigyDrive" specified as the"
        SAY "TonigyDrive in "arguments.1" doesn't exist"
        SAY "Use /? for other help"
        EXIT
        END

IF nocddb = 1 THEN
   DO
   SAY 'not quite implemented yet...'
   EXIT
   END

/* Get the cddb entry for the CD and write it to a temp file */
'querycddb.exe 'CDdrive User UserHost cddbHost cddbPort' 1>cddbxxxx.tmp'

I=0
DO WHILE Lines(cddbxxxx.tmp)
        line.I = SubWord(LineIn(cddbxxxx.tmp),1)
        I=I+1
END

IF keepTempFile=0 THEN
        DO
        /* Close the temp file and delete it */
        rc=Stream('cddbxxxx.tmp', C, CLOSE)
        rc=SysFileDelete('cddbxxxx.tmp')
        END

/* If disc wasn't found, exit */
IF I=0 THEN
DO
        say
        say
        say "Error. Either the disc wasn't found on the cddb server,"
        say "or there was a problem with the cddb server"
        EXIT
END

/* logic if there was a dupe cddb entry */
IF SubWord(line.1, 1, 1) = 'Artist:' THEN
DO
Tracks=I-5
CDArtist=SubWord(line.1,2)
CDTitle=SubWord(line.2,2)
CDCategory=SubWord(line.3,2)
ADD=4
END

ELSE
DO
Tracks=I-6
CDArtist=SubWord(line.2,2)
CDTitle=SubWord(line.3,2)
CDCategory=SubWord(line.4,2)
ADD=5
END

say

J=1
DO Tracks
        K=J+ADD
        Track.J=SubWord(line.K,2)
        /*say Track.J*/
        J=J+1
END

IF copy = 1 | domp3 = 1 THEN
        DO
        /* Make the proper directories.  If they already exist, fail. */

        /* try to find artist dir */
        IF(SubStr(mp3dir,LENGTH(mp3dir),1))='\' THEN
                DO
                mp3dir=SubStr(mp3dir,1,LENGTH(mp3dir)-1)
                END

        CDArtistDir=Translate(CDArtist, outtranslate, intranslate)
        ArtistDir=mp3Dir'\'CDArtistdir
        newdir=DIRECTORY(mp3Dir)

        /* see if artist dir already exists */
        rc=SysFileTree(CDArtist,stem,D)
        IF stem.0=0 THEN
                DO
                rc=SysMkDir(ArtistDir)
                IF rc>0 THEN
                        DO
                        SAY 'rc= 'rc
                        SAY "Failed to create Artist Directory in the directory"
                        SAY ArtistDir
                        SAY "check the tutil.opt file"
                        newdir=DIRECTORY(curdir)
                        EXIT
                        END
                END
        /* try to find album dir */
        newdir=DIRECTORY(ArtistDir)
        CDTitledir=Translate(CDTitle, outtranslate, intranslate)
        AlbumDir=ArtistDir'\'CDTitledir

        /* see if artist dir already exists */
        rc=SysFileTree(CDTitle,stem,D)
        IF stem.0=0 THEN
                DO

                rc=SysMkDir(AlbumDir)
                IF rc>0 THEN
                        DO
                        SAY 'rc= 'rc
                        SAY "Failed to create Album Directory in the"
                        SAY "directory "AlbumDir
                        newdir=DIRECTORY(curdir)
                        EXIT
                        END
                END
        END

/* Now do the Loop to rip the tracks */

H=1

/* If /ask was specified, ask which track to encode */
IF ask=1 THEN
   DO
   say
   say 'The number of tracks on this CD is 'Tracks'.'
   say 'Enter the track number or range you want to encode or copy.'
   say 'For example:  4-9.'
   pull trackresponse
   PARSE VAR trackresponse begintrack '-' endtrack
   tracktoenc=SubWord(begintrack,1,1)
   IF (DATATYPE(tracktoenc,'W'))<>1 THEN
      DO
      say 'invalid tracknumber'
      rc=DIRECTORY(curdir)
      EXIT
      END
   IF tracktoenc>Tracks THEN
      DO
      say 'track out of range'
      rc=DIRECTORY(curdir)
      EXIT
      END
   IF SubStr(tracktoenc,1,1)='0' THEN
      DO
      say "don't put a '0' in front of the track number"
      rc=DIRECTORY(curdir)
      EXIT
      END

   H=tracktoenc

   IF endtrack <> '' THEN
   DO
      tracktoenc=SubWord(endtrack,1,1)
      IF (DATATYPE(tracktoenc,'W'))<>1 THEN
         DO
         say 'invalid tracknumber'
         rc=DIRECTORY(curdir)
         EXIT
         END
      IF tracktoenc>Tracks THEN
         DO
         say 'track out of range'
         rc=DIRECTORY(curdir)
         EXIT
         END
      IF SubStr(tracktoenc,1,1)='0' THEN
         DO
         say "don't put a '0' in front of the track number"
         rc=DIRECTORY(curdir)
         EXIT
         END
      Tracks=tracktoenc-H+1
      END

   ELSE
   DO
   Tracks=1
   END

   IF tracks<1 THEN
      DO
      say 'starting track must be before ending track'
      rc=DIRECTORY(curdir)
      EXIT
      END
END

DO Tracks
        /* Properly format the track number */
        IF H<10 THEN
                tracknum='0'H
                ELSE
                tracknum=H

        /* Get the wav file size */
        savedir=DIRECTORY()
        newdir=DIRECTORY(TonigyDrive'\')
        wavfile='track'tracknum'.wav'
        rc=SysFileTree(wavfile,'wavstem.',F)
        wavfilelength=SubWord(wavstem.1,3,1)

        /* Calculate the time of the track.  Might be a bit off, if there's extra info in the RIFF header, but since it's only accurate to the second, I don't care. */
        seconds=(wavfilelength-44)%(4*44100)
        minutes=seconds%60
        leftoverseconds=seconds-(minutes*60)

        IF leftoverseconds<10 THEN
                leftoverseconds='0'leftoverseconds

        tracktime=minutes':'leftoverseconds

        rc=DIRECTORY(savedir)

        Infile='"'TonigyDrive'\track'tracknum'.wav"'
        Trackfile=Translate(Track.H, outtranslate, intranslate)
        Outfile='"'AlbumDir'\'tracknum'-'trackfile'.mp3"' /* Put the track number at the beginning of the filename */
        Outwavfile='"'tracknum'-'trackfile'.wav"'

        /* ID3 v1 is limited to 30 characters */

        IF Length(CDArtist)>30 THEN
                ID3Artist=SubStr(CDArtist, 1, 30)
                ELSE
                ID3Artist=CDArtist

        IF Length(CDTitle)>30 THEN
                ID3Title=SubStr(CDTitle, 1, 30)
                ELSE
                ID3Title=CDTitle

        IF Length(Track.H)>30 THEN
                ID3Track=SubStr(Track.H, 1, 30)
                ELSE
                ID3Track=Track.H

        /* Put the track number in the ID3 comment field */

        ID3Comment='Track 'tracknum

        IF doEAs = 1 THEN
                DO
                /* Write EA's to Tonigy drive */
                leng=Right(D2C(Length(Track.H)+4), 2, '00'x)
                longname=Reverse('FFFD'x)||Reverse(leng)||Track.H||'.wav'
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', '.LONGNAME', longname)

                leng=Right(D2C(Length(CDArtist)), 2, '00'x)
                CDARTISTEA=Reverse('FFFD'x)||Reverse(leng)||CDArtist
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', 'CDARTIST', CDARTISTEA)

                leng=Right(D2C(Length(CDTitle)), 2, '00'x)
                CDTITLEEA=Reverse('FFFD'x)||Reverse(leng)||CDTitle
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', 'CDTITLE', CDTITLEEA)

                leng=Right(D2C(Length(tracknum)), 2, '00'x)
                TRACKNUMBEREA=Reverse('FFFD'x)||Reverse(leng)||tracknum
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', 'TRACKNUMBER', TRACKNUMBEREA)

                leng=Right(D2C(Length(Track.H)), 2, '00'x)
                TRACKTITLEEA=Reverse('FFFD'x)||Reverse(leng)||Track.H
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', 'TRACKTITLE', TRACKTITLEEA)

                leng=Right(D2C(Length(CDCategory)), 2, '00'x)
                CATEGORYEA=Reverse('FFFD'x)||Reverse(leng)||CDCategory
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', 'CATEGORY', CATEGORYEA)

                leng=Right(D2C(Length(tracktime)), 2, '00'x)
                TIMEEA=Reverse('FFFD'x)||Reverse(leng)||tracktime
                rc=SysPutEA(TonigyDrive'\track'tracknum'.wav', 'TIME', TIMEEA)

                END /* End of doEAs */

        IF domp3 = 1 THEN
        DO
        newdir=DIRECTORY(AlbumDir)

        IF nogenre = 1 THEN
            DO
            tempcategory=CDCategory
            CDCategory=''
            END

        /* make the lame command line */
        mp3file=tracknum'-'trackfile'.mp3'
        rc=SysFileTree(mp3file, mp3exists, FO)

        /* If the mp3 file exists, skip using lame to create it */
        IF mp3exists.0=0 THEN
        DO
        'lame 'LameOpts '--tt "'ID3Track'" --tg "'CDCategory'" --ta "'ID3Artist'" --tl "'ID3Title'" --tc "'ID3Comment'" --tn 'tracknum Infile Outfile
        END

        IF nogenre = 1 THEN
            DO
            CDCategory=tempcategory
            END

        IF doEAs = 1 THEN
                DO
                MP3file=tracknum'-'Trackfile'.mp3'
                /* Write EA's to mp3 */
                leng=Right(D2C(Length(MP3file)), 2, '00'x)
                MP3longname=tracknum'-'Track.H'.mp3'  /* So the longname EA has the -untranslated- characters */
                longname=Reverse('FFFD'x)||Reverse(leng)||MP3longname
                rc=SysPutEA(MP3file, '.LONGNAME', longname)

                leng=Right(D2C(Length(CDArtist)), 2, '00'x)
                CDARTISTEA=Reverse('FFFD'x)||Reverse(leng)||CDArtist
                rc=SysPutEA(MP3file, 'CDARTIST', CDARTISTEA)

                leng=Right(D2C(Length(CDTitle)), 2, '00'x)
                CDTITLEEA=Reverse('FFFD'x)||Reverse(leng)||CDTitle
                rc=SysPutEA(MP3file, 'CDTITLE', CDTITLEEA)

                leng=Right(D2C(Length(tracknum)), 2, '00'x)
                TRACKNUMBEREA=Reverse('FFFD'x)||Reverse(leng)||tracknum
                rc=SysPutEA(MP3file, 'TRACKNUMBER', TRACKNUMBEREA)

                leng=Right(D2C(Length(Track.H)), 2, '00'x)
                TRACKTITLEEA=Reverse('FFFD'x)||Reverse(leng)||Track.H
                rc=SysPutEA(MP3file, 'TRACKTITLE', TRACKTITLEEA)

                leng=Right(D2C(Length(CDCategory)), 2, '00'x)
                CATEGORYEA=Reverse('FFFD'x)||Reverse(leng)||CDCategory
                rc=SysPutEA(MP3file, 'CATEGORY', CATEGORYEA)

                leng=Right(D2C(Length(tracktime)), 2, '00'x)
                TIMEEA=Reverse('FFFD'x)||Reverse(leng)||tracktime
                rc=SysPutEA(MP3file, 'TIME', TIMEEA)

                END /* End of doEAs */

        END /* End of domp3 */

        IF copy = 1 THEN
                DO
                copydir=DIRECTORY()
                'xcopy 'Infile '"'AlbumDir'"'
                rc=DIRECTORY(AlbumDir)

                'rename track'tracknum'.wav' outwavfile
                rc=DIRECTORY(copydir)
                END

        H=H+1 /* Increment to the next track */
END

/* restore the original directory */
newdir=DIRECTORY(curdir)
say
say 'All Done.'
EXIT

/* The end of the program */

/* format of what's returned by querycddb.exe:   */
/*                                                                  */
/*  DiscID: e4121f10   */
/*  Artist: Peter Gabriel   */
/*  Title: Shaking The Tree - Sixteen Golden Greats   */
/*  Category: rock   */
/*                                                                     */
/*  Track01: Solsbury Hill   */
/*  Track02: I Don't Remember   */
/*    (the rest of the tracks...)   */
/*  Track16: Biko   */

/* If there's a duplicate cddb entry, after you select one, you get:    */
/*                                              */
/*     DiscID of this CD: 460af506    */
/*     Choosen DiscID: 4a0af506    */
/*     Artist: Pink Floyd     */
/*     Title: Meddle    */
/*     Category: rock    */
/*                                                */
/*     Track01: One Of These Days    */
/*     Track02: A Pillow Of Winds    */
/*     Track03: Fearless    */
/*     Track04: San Tropez    */
/*     Track05: Seamus    */
/*     Track06: Echoes    */

/*  If the disc isn't found, then cddbxxxx.tmp will be blank, but still there.  */
